@@ -9,9 +9,12 @@ import socket
 import traceback
 import os
 import sys
+import aiomysql
+import asyncio
 import pymysql
 import re
 import configparser
+import urllib.parse
 from loguru import logger
 import hashlib
 import platform
@@ -21,18 +24,23 @@ import string
 import queue
 import threading
 import zmq
+import shutil
+from sys import argv
+import sys
+import os
+import shutil
+import glob
+from typing import Any, Optional, Tuple, List, Dict
+import functools
+import inspect
+import uuid
+from PIL import Image
+import io
 
 # reload(sys)
 # sys.setdefaultencoding('utf8')
 # logger.info("WARN:please configuration the configuraion_file variable")
 
-configuration_file = "/data/apps/public/conf.ini"
-configuration_path_prefix = "/data"
-system = platform.system()
-if system == "Darwin":
-    configuration_path_prefix = os.path.expanduser("~")
-configuration_file = "{}/apps/public/conf.ini".format(configuration_path_prefix)
-
 
 def getConfig(config_file=None):
     warnings.warn("此方法已废弃，不推荐使用，请使用 get_config 替换。", DeprecationWarning)
@@ -46,54 +54,15 @@ def execute_command(cmd):
 
 def get_config(config_file=None):
     if config_file is None:
-        config_file = configuration_file
-    logger.info(configuration_file)
+        configuration_path_prefix = os.path.expanduser("~")
+        config_file = "{}/apps/public/conf.ini".format(configuration_path_prefix)
+
+    logger.info(config_file)
     config = configparser.ConfigParser()
     config.read(config_file)
     return config
 
 
-# for python2
-# def execute_command2(cmd):
-#     status,output = commands.getstatusoutput(cmd)
-#     if status == 0:
-#         status = True
-#     else:
-#         status = False
-#
-#     tempLines = output.split("\n")
-#
-#     lines = []
-#
-#     for e in tempLines:
-#         t = e.strip()
-#         if len(t) >0 :
-#             lines.append(t)
-#
-#     return (status, output, lines)
-
-
-# for python3
-def execute_command3(cmd):
-    warnings.warn("此方法已废弃，不推荐使用，请使用 command 替换。", DeprecationWarning)
-    status, output = subprocess.getstatusoutput(cmd)
-    if status == 0:
-        status = True
-    else:
-        status = False
-
-    temp_lines = output.split("\n")
-
-    lines = []
-
-    for e in temp_lines:
-        t = e.strip()
-        if len(t) > 0:
-            lines.append(t)
-
-    return status, output, lines
-
-
 def command(cmd):
     """执行shell命令,并返回结果
 
@@ -136,34 +105,6 @@ def set_logger_level(level):
     return handler_id
 
 
-def parseCommandArguments(args):
-    arg_dict = {}
-    for argument in args:
-        logger.debug("argument:" + argument)
-        if argument.startswith("-D"):
-            arg = argument[2:]
-
-            idx = arg.find("=")
-
-            k = arg[0:idx]
-            v = arg[idx + 1:]
-            arg_dict[k] = v
-    return arg_dict
-
-
-def wait_until(future_timestamp):
-    logger.debug("futureTimestamp %s " % future_timestamp)
-
-    now = datetime_tool.datetime.now()
-    now_timestamp = timestamp(now)
-
-    logger.debug("nowTimestamp %s " % now_timestamp)
-
-    if now_timestamp < future_timestamp:
-        d = future_timestamp - now_timestamp
-        timetool.sleep(d)
-
-
 # 获取redisCluster的对象
 # def getRedisCluster(redisClusterStr):
 #     from rediscluster import StrictRedisCluster
@@ -178,138 +119,6 @@ def wait_until(future_timestamp):
 #     rc = StrictRedisCluster(startup_nodes=redisCluster, decode_responses=True)
 #     return rc
 
-# mysql some operations
-def getDatabase(section="mysql"):
-    try:
-        config = configparser.ConfigParser()
-        config.read(configuration_file)
-        host = config.get(section, "host")
-        port = int(config.get(section, "port"))
-        user = config.get(section, "user")
-        password = config.get(section, "passwd")
-        database = config.get(section, "database")
-        charset = config.get(section, "charset")
-
-        db = pymysql.connect(host=host, port=port, user=user, passwd=password, db=database, charset=charset,
-                             autocommit=True,
-                             cursorclass=pymysql.cursors.DictCursor)
-        return db
-    except ImportError:
-        logger.error("Error: configparser or pymysql module not exists")
-    except Exception as e:
-        logger.error(e)
-        traceback.print_exc()
-
-    return None
-
-
-def query(database, sql, argumentTuple=(), timestamp2str=True):
-    cursor = database.cursor()
-    logger.debug("query sql:\t%s, arguments: %s" % (sql, argumentTuple))
-    if len(argumentTuple) == 0:
-        cursor.execute(sql)
-    else:
-        cursor.execute(sql, argumentTuple)
-    rows = []
-
-    if timestamp2str:
-        timestamp_field_array = []
-        date_field_array = []
-        for tp in cursor.description:
-            if tp[1] == pymysql.constants.FIELD_TYPE.TIMESTAMP or tp[1] == pymysql.constants.FIELD_TYPE.DATETIME:
-                timestamp_field_array.append(tp[0])
-            elif tp[1] == pymysql.constants.FIELD_TYPE.DATE:
-                date_field_array.append(tp[0])
-
-        for row in cursor:
-            for field in timestamp_field_array:
-                if row[field] is not None:
-                    tmp_value = row[field].strftime("%Y-%m-%d %H:%M:%S")
-                    row[field] = tmp_value
-
-            for field in date_field_array:
-                if row[field] is not None:
-                    tmp_value = row[field].strftime("%Y-%m-%d")
-                    row[field] = tmp_value
-            rows.append(row)
-    else:
-        for row in cursor:
-            rows.append(row)
-    return rows
-
-
-def insert(database, tableName, dic, commit=True):
-    cursor = database.cursor()
-    cols = []
-    vals = []
-    placeholders = []
-    doc_id = ""
-    for key in dic.keys():
-        val = dic[key]
-        if val is not None:
-            cols.append(key)
-            placeholders.append("%s")
-            vals.append(val)
-    insert_sql = "INSERT INTO " + tableName + " ( %s ) VALUES ( %s )" % (",".join(cols), ",".join(placeholders))
-    logger.debug(insert_sql)
-
-    if commit:
-        logger.info(tuple(vals))
-        cursor.execute(insert_sql, tuple(vals))
-        doc_id = cursor.lastrowid
-    if commit:
-        database.commit()
-
-    return doc_id
-
-
-def updateById(database, tableName, id, dic, idFieldName="id", commit=True):
-    cursor = database.cursor()
-    vals = []
-    placeholders = []
-    for key in dic.keys():
-        val = dic[key]
-        if val is not None:
-            placeholders.append("{} = %s ".format(key))
-            vals.append(val)
-    setting = " , ".join(placeholders)
-    vals.append(id)
-    update_sql = "update {0} set {1} where {2} = %s ".format(tableName, setting, idFieldName)
-    logger.debug(update_sql)
-
-    if commit:
-        logger.debug(tuple(vals))
-        cursor.execute(update_sql, tuple(vals))
-    if commit:
-        database.commit()
-
-
-def execute(database, sql, argumentTuple=()):
-    cursor = database.cursor()
-    logger.debug("query sql:\t%s, arguments: %s" % (sql, argumentTuple))
-    if len(argumentTuple) == 0:
-        cursor.execute(sql)
-    else:
-        cursor.execute(sql, argumentTuple)
-
-
-def delete(database, sql, commit=True):
-    cursor = database.cursor()
-    logger.debug("delete sql:\t" + sql)
-    if commit:
-        cursor.execute(sql)
-        database.commit()
-
-
-def mysql_execute(database, sql, argumentTuple, commit=True):
-    cursor = database.cursor()
-    if argumentTuple:
-        cursor.execute(sql, argumentTuple)
-    else:
-        cursor.execute(sql)
-    if commit:
-        database.commit()
-
 
 # 获取文件的创建时间
 def getFileCreateTime(filePath):
@@ -360,6 +169,8 @@ def getFilePrefix(path):
 def getFilePostfix(path):
     return os.path.splitext(path)[1][1:]
 
+def getFileExtension(path):
+    return os.path.splitext(path)[1][1:]
 
 def getLocalHostname():
     hostname = ""
@@ -495,10 +306,6 @@ def wecom_send_text(title,message, token=None):
                   auth=('Content-Type', 'application/json'))
 
 
-def get_md5_code(arg, is_file=False):
-    return md5(arg, is_file=is_file)
-
-
 def md5(arg, is_file=False):
     """
     :param arg: 原始字符串，或者文件路径
@@ -584,6 +391,13 @@ def check_requirement(requirement_list):
                 logger.info("{0} is missing, so the program exits!".format(package))
                 exit(-1)
 
+def get_domain(url):
+    '''
+    获取域名，包含端口，例如：https://www.baidu.com:8080/path/to/page ，解析后为 www.baidu.com:8080
+    '''
+    parsed_url = urllib.parse.urlparse(url)
+    domain = parsed_url.netloc
+    return domain
 
 # reconnecting mysql , 参考 https://www.kingname.info/2017/04/17/decorate-for-method/
 def pingmysql(original_function):
@@ -613,21 +427,24 @@ def pingmysql(original_function):
 
 class Mysql():
 
-    def __init__(self, configuraion_file=configuration_file, section="mysql"):
-        self.default_database_name = ''
+    def __init__(self, configuration_file=None, section="mysql",host=None, port=3306, user=None, password=None,database_name=None, charset="utf8"):
+        self.default_database_name = database_name
         self.database = None
         self.columns_dict = None
         self.meta_update_time = 0  # self.columns_dict数据的更新时间，self.columns_dict 每5分钟左右更新一次
         try:
-            config = configparser.ConfigParser()
-            config.read(configuraion_file)
-            host = config.get(section, "host")
-            port = int(config.get(section, "port"))
-            user = config.get(section, "user")
-            password = config.get(section, "passwd")
-            self.default_database_name = config.get(section, "database")
-            charset = config.get(section, "charset")
-
+            if host is None:
+                logger.info(configuration_file)
+                config = get_config(configuration_file)
+                host = config.get(section, "host")
+                port = int(config.get(section, "port"))
+                user = config.get(section, "user")
+                password = config.get(section, "passwd")
+                self.default_database_name = config.get(section, "database")
+                charset = config.get(section, "charset")
+            
+            port = int(port)
+            logger.info(f"host: {host}, port: {port}, user: {user}, password: {password}, database: {self.default_database_name}, charset: {charset}")
             self.database = pymysql.connect(host=host, port=port, user=user, passwd=password,
                                             db=self.default_database_name,
                                             charset=charset, autocommit=True,
@@ -796,6 +613,11 @@ class Mysql():
             self.database.commit()
 
         return id
+    
+    @pingmysql
+    def update(self, tableName, dic, idFieldName="id", commit=True):
+        doc_id = dic.get(idFieldName)
+        return self.update_by_id(tableName, doc_id, dic, idFieldName, commit)
 
     @pingmysql
     def update_by_id(self, tableName, id, dic, idFieldName="id", commit=True):
@@ -896,6 +718,206 @@ class Mysql():
             logger.debug(values_array[start:start + batch_size])
             cursor.executemany(ql, values_array[start:start + batch_size])
             self.database.commit()
+            
+def async_pingmysql(func):
+    @functools.wraps(func)
+    async def wrapper(self, *args, **kwargs):
+        if hasattr(self, "refresh_columns_dict") and inspect.iscoroutinefunction(self.refresh_columns_dict):
+            await self.refresh_columns_dict()
+        return await func(self, *args, **kwargs)
+    return wrapper
+
+
+class AsyncMysql:
+    def __init__(self, host, port, user, password, db,
+                 charset="utf8", minsize=1, maxsize=10, loop=None):
+        self.default_database_name = db
+        self.pool = None
+        self.columns_dict = None
+        self.meta_update_time = 0
+        self.loop = loop or asyncio.get_event_loop()
+        self.db_config = {
+            "host": host,
+            "port": port,
+            "user": user,
+            "password": password,
+            "db": db,
+            "charset": charset,
+            "minsize": minsize,
+            "maxsize": maxsize,
+        }
+
+    @classmethod
+    async def create(cls, **kwargs):
+        instance = cls(**kwargs)
+        await instance.init_pool()
+        return instance
+
+    async def init_pool(self):
+        self.pool = await aiomysql.create_pool(
+            loop=self.loop,
+            cursorclass=aiomysql.DictCursor,
+            autocommit=True,
+            **self.db_config
+        )
+
+    async def refresh_columns_dict(self):
+        if self.columns_dict is None or timetool.time() - self.meta_update_time > 300:
+            try:
+                async with self.pool.acquire() as conn:
+                    async with conn.cursor() as cursor:
+                        sql = """
+                            SELECT TABLE_SCHEMA, TABLE_NAME, COLUMN_NAME, DATA_TYPE
+                            FROM information_schema.columns
+                            WHERE TABLE_SCHEMA NOT IN ('information_schema','performance_schema','mysql','sys')
+                        """
+                        await cursor.execute(sql)
+                        rows = await cursor.fetchall()
+                        self.columns_dict = {
+                            f"{row['TABLE_SCHEMA']}.{row['TABLE_NAME']}.{row['COLUMN_NAME']}": row
+                            for row in rows
+                        }
+                        self.meta_update_time = timetool.time()
+            except Exception:
+                traceback.print_exc()
+
+    async def is_legal_value(self, table_name: str, column_name: str, val: Any) -> bool:
+        if val is None:
+            return True
+        if "." not in table_name:
+            table_name = f"{self.default_database_name}.{table_name}"
+        col_name = f"{table_name}.{column_name}"
+        await self.refresh_columns_dict()
+        row = self.columns_dict.get(col_name)
+        if row is None:
+            logger.warning(f"{self.columns_dict} 中不包含 {col_name}")
+            return False
+        data_type = row.get("DATA_TYPE")
+        if data_type in ['int', 'tinyint', 'bigint']:
+            str_val = str(val)
+            if not str_val.lstrip('+-').isnumeric():
+                logger.error(f"{column_name} = {val} 不是合法的 {data_type}")
+                return False
+        return True
+
+    @async_pingmysql
+    async def query(self, sql: str, args: Tuple = (), timestamp2str: bool = True):
+        async with self.pool.acquire() as conn:
+            async with conn.cursor() as cursor:
+                logger.debug("query sql:\t%s, arguments: %s", sql, args)
+                await cursor.execute(sql, args)
+                rows = await cursor.fetchall()
+                if timestamp2str:
+                    for row in rows:
+                        for k, v in row.items():
+                            if hasattr(v, "strftime"):
+                                row[k] = v.strftime("%Y-%m-%d %H:%M:%S")
+                    return rows
+                return rows
+
+    @async_pingmysql
+    async def get(self, sql: str, args: Tuple = (), timestamp2str: bool = True):
+        if "limit" not in sql.lower():
+            sql += " limit 1"
+        rows = await self.query(sql, args, timestamp2str)
+        return rows[0] if rows else None
+
+    @async_pingmysql
+    async def value(self, sql: str, args: Tuple = ()): 
+        row = await self.get(sql, args)
+        return next(iter(row.values())) if row else None
+
+    @async_pingmysql
+    async def get_int(self, sql: str, args: Tuple = ()): 
+        val = await self.value(sql, args)
+        return int(val) if val is not None else None
+
+    @async_pingmysql
+    async def get_str(self, sql: str, args: Tuple = ()): 
+        val = await self.value(sql, args)
+        return str(val) if val is not None else None
+
+    @async_pingmysql
+    async def get_bool(self, sql: str, args: Tuple = ()): 
+        val = await self.value(sql, args)
+        return bool(val) if val is not None else None
+
+    @async_pingmysql
+    async def insert(self, table_name: str, dic: dict, commit: bool = True):
+        async with self.pool.acquire() as conn:
+            async with conn.cursor() as cursor:
+                cols, vals, placeholders = [], [], []
+                for key, val in dic.items():
+                    if val is not None and await self.is_legal_value(table_name, key, val):
+                        cols.append(key)
+                        placeholders.append("%s")
+                        vals.append(val)
+                sql = f"INSERT INTO {table_name} ({','.join(cols)}) VALUES ({','.join(placeholders)})"
+                logger.debug(sql)
+                logger.debug(vals)
+                await cursor.execute(sql, vals)
+                if commit:
+                    await conn.commit()
+                return cursor.lastrowid
+
+    @async_pingmysql
+    async def update_by_id(self, table_name: str, id_val, dic: dict, id_field_name="id", commit: bool = True):
+        async with self.pool.acquire() as conn:
+            async with conn.cursor() as cursor:
+                placeholders, vals = [], []
+                for key, val in dic.items():
+                    if val is None:
+                        placeholders.append(f"{key} = NULL")
+                    elif await self.is_legal_value(table_name, key, val):
+                        placeholders.append(f"{key} = %s")
+                        vals.append(val)
+                vals.append(id_val)
+                setting = ", ".join(placeholders)
+                sql = f"UPDATE {table_name} SET {setting} WHERE {id_field_name} = %s"
+                logger.debug(sql)
+                logger.debug(vals)
+                await cursor.execute(sql, vals)
+                if commit:
+                    await conn.commit()
+                return cursor.rowcount
+
+    @async_pingmysql
+    async def update(self, table_name: str, dic: dict, id_field_name: str = "id", commit: bool = True):
+        id_val = dic.get(id_field_name)
+        if id_val is None:
+            raise ValueError(f"{id_field_name} is required in update()")
+        return await self.update_by_id(table_name, id_val, dic, id_field_name, commit)
+
+    @async_pingmysql
+    async def execute(self, sql: str, args: Tuple = (), commit: bool = True):
+        async with self.pool.acquire() as conn:
+            async with conn.cursor() as cursor:
+                logger.debug("execute sql:\t%s, arguments: %s", sql, args)
+                await cursor.execute(sql, args)
+                if commit:
+                    await conn.commit()
+                return cursor.rowcount
+
+    @async_pingmysql
+    async def delete(self, sql: str, args: Tuple = (), commit: bool = True):
+        logger.debug("delete sql:\t%s, args: %s", sql, args)
+        return await self.execute(sql, args, commit)
+
+    @async_pingmysql
+    async def bulk_insert(self, table_name: str, data: List[Dict], batch_size: int = 100):
+        if not data:
+            return
+        field_array = list(data[0].keys())
+        placeholder = ",".join(["%s"] * len(field_array))
+        field_line = ",".join(field_array)
+        sql = f"INSERT INTO {table_name} ({field_line}) VALUES ({placeholder})"
+        values_array = [[row.get(field) for field in field_array] for row in data]
+        async with self.pool.acquire() as conn:
+            async with conn.cursor() as cursor:
+                for start in range(0, len(values_array), batch_size):
+                    batch = values_array[start:start + batch_size]
+                    await cursor.executemany(sql, batch)
+                    await conn.commit()
 
 
 class ElasticSearch():
@@ -950,6 +972,20 @@ class ElasticSearch():
         return requests.get(url="{}{}".format(self.urlPrefix, action), auth=(self.username, self.password),
                             headers=self.headers, data=data1).json()
 
+    def get(self, index, doc_id):
+        """
+        :param index:
+        :param doc_id:
+        :return:
+        """
+        action = f"/{index}/_doc/{doc_id}"
+        doc = requests.get(url="{}{}".format(self.urlPrefix, action), auth=(self.username, self.password),
+                            headers=self.headers).json()
+        found = doc.get("found")
+        if found == False:
+            return None
+        return doc["_source"]
+
     def query(self, action, query_body=None):
         """
         :param action: 比如: /scene_model/_search
@@ -1029,7 +1065,6 @@ class ElasticSearch():
 
     def put(self, action, data=None):
         """
-
         :param action:
         :param data:
         :return:
@@ -1049,8 +1084,18 @@ class ElasticSearch():
         :return:
         """
         return self.post("/{}/_update/{}".format(index_name, doc_id), data={"doc": data})
+    
+    def upsert(self, index_name, doc_id, json,):
+        """
+        :param index_name : 索引的名称
+        :param doc_id : 文档的id
+        :param json: 格式为{'id': '332214', 'price': '10', 'sort': '5', 'is_use': '3'}
+        :return:
+        """
 
-
+        return self.post("/{}/_update/{}".format(index_name, doc_id), data={"doc": json, "doc_as_upsert": True})
+         
+         
 def is_number(str):
     """
     判断是否为数字，整数或者小数都可以
@@ -1071,6 +1116,7 @@ class AppleScriptTool():
         self.begin = f"""tell application "System Events"
     tell process "{name}"
         set frontmost to true """
+        self.all_cmd = ""
 
 
     end = """end tell
@@ -1099,28 +1145,48 @@ end tell"""
         """
         subprocess.run(["osascript", "-e", cmd])
 
-    def delay_until(self, text):
-        return f"""
-            repeat until exists window "{text}"
+    def before_return(self,cmd):
+        self.all_cmd = f"{self.all_cmd}\n{cmd}"
+        return cmd
+
+    def delay_until(self, element):
+        cmd = f"""
+            repeat until {element}
             end repeat
         """
+        return self.before_return(cmd)
+    
+    def click(self,element):
+        cmd = f"""click {element}"""
+        return self.before_return(cmd)
 
     def input(self, text):
-        return f"""
+        cmd = f"""
             set the clipboard to "{text}"
             tell application "System Events" to keystroke "v" using command down
         """
+        return self.before_return(cmd)
 
-    press_enter = "keystroke return"
+    def press_enter(self):
+        cmd = "keystroke return"
+        return self.before_return(cmd)
 
-    press_down = "keystroke (ASCII character 31)"
+    def press_down(self):
+        cmd = "keystroke (ASCII character 31)"
+        return self.before_return(cmd)
+    
+    def set_value(self,text_field,text):
+        cmd = f"""  set theTextField to {text_field}
+                    set value of theTextField to "{text}"
+        """
+        return self.before_return(cmd)
 
     def construct_cmd(self, cmd):
         cmd1 = f"""{self.begin}
                         {cmd}
        {self.end}"""
-        cmd1 = cmd1.replace("press_enter", self.press_enter)
-        cmd1 = cmd1.replace("press_down", self.press_down)
+        cmd1 = cmd1.replace("press_enter", "keystroke return")
+        cmd1 = cmd1.replace("press_down", "keystroke (ASCII character 31)")
         cmd1 = re.sub(r"""input (.+)""",
                       r"""set the clipboard to "\1" \n        tell application "System Events" to keystroke "v" using command down""",
                       cmd1)
@@ -1146,14 +1212,34 @@ end tell"""
         self.show()
         subprocess.run(["osascript", "-e", whole_cmd ])
 
-    def copy(self,message):
-        return f"""set the clipboard to "{message}" """
+    def submit(self):
+        cmd = self.construct_cmd(self.all_cmd)
+        self.all_cmd = ""
+        self.show()
+        logger.info(cmd)
+        subprocess.run(["osascript", "-e", cmd ])
 
-    def paste(self):
-        return """tell application "System Events" to keystroke "v" using command down""" 
+
+    def copy(self,message,execute=False):
+        cmd = f"""set the clipboard to "{message}" """
+        if execute:
+            subprocess.run(["osascript", "-e", cmd ])
+        else:
+            return self.before_return(cmd)
+
+    def paste(self,execute=False):
+        cmd = """tell application "System Events" to keystroke "v" using command down""" 
+        if execute:
+            subprocess.run(["osascript", "-e", cmd ])
+        else:
+            return self.before_return(cmd)
     
+    def sleep(self,seconds=1):
+        cmd = f"delay {seconds}"
+        return self.before_return(cmd)
+        
     def get_loc_size(self):
-        return f"""
+        cmd = f"""
             tell application "System Events"
                 tell application process "{self.name}"
                     -- 获取窗口的左上角位置和大小
@@ -1168,34 +1254,38 @@ end tell"""
             set width to item 1 of windowSize
             set height to item 2 of windowSize
             """
+        return self.before_return(cmd)
     
     def screenshot(self):
         """
         截图并将截图保存到剪贴板
         """
         cmd = self.get_loc_size()
-        return f"""{cmd}\n do shell script "screencapture -c -R" & x & "," & y & "," & width & "," & height & " " """
+        cmd = f"""{cmd}\n do shell script "screencapture -c -R" & x & "," & y & "," & width & "," & height & " " """
+        return self.before_return(cmd)
 
     def screenshot_to(self,path):
         cmd = self.get_loc_size()
-        return f"""{cmd}\n do shell script "screencapture -R" & x & "," & y & "," & width & "," & height & " " & quoted form of "{path}" """
+        cmd = f"""{cmd}\n do shell script "screencapture -R" & x & "," & y & "," & width & "," & height & " " & quoted form of "{path}" """
+        return self.before_return(cmd)
 
 class MqServer():
-
     # 创建一个阻塞队列
     message_queue = queue.Queue(maxsize=5000)  # 可以设置队列最大容量
 
-    def __init__(self,publish_port=20055,receive_port=20065):
+    def __init__(self, publish_port=20055, receive_port=20065):
         self.publish_port = publish_port
         self.receive_port = receive_port
+        self.context = zmq.Context()  # 共享同一个 Context
+        self.publish_socket = None   # 发布端的 socket
+        self.receive_socket = None   # 接收端的 socket
 
     def start_publish(self):
-        context = zmq.Context()
-        socket = context.socket(zmq.PUB)
-        socket.setsockopt(zmq.HEARTBEAT_IVL, 2000)      # 发送心跳的间隔（毫秒）
-        socket.setsockopt(zmq.HEARTBEAT_TIMEOUT, 6000)  # 超过此时间未收到心跳，认为断开（毫秒）
-        socket.setsockopt(zmq.HEARTBEAT_TTL, 3000)      # 客户端发送心跳的间隔时间（毫秒）
-        socket.bind(f"tcp://*:{self.publish_port}")
+        self.publish_socket = self.context.socket(zmq.PUB)
+        self.publish_socket.setsockopt(zmq.HEARTBEAT_IVL, 2000)      # 发送心跳的间隔（毫秒）
+        self.publish_socket.setsockopt(zmq.HEARTBEAT_TIMEOUT, 6000)  # 超过此时间未收到心跳，认为断开（毫秒）
+        self.publish_socket.setsockopt(zmq.HEARTBEAT_TTL, 3000)      # 客户端发送心跳的间隔时间（毫秒）
+        self.publish_socket.bind(f"tcp://*:{self.publish_port}")
 
         while True:
             msg = self.message_queue.get()  # get() 会在队列为空时阻塞
@@ -1204,31 +1294,46 @@ class MqServer():
 
             if msg == 'exit':
                 sys.exit()
-            socket.send(msg.encode('utf-8'))
+            self.publish_socket.send(msg.encode('utf-8'))
             timetool.sleep(1)
 
     def start_receive(self):
-        context = zmq.Context()
-        socket = context.socket(zmq.REP)
-        socket.bind(f"tcp://*:{self.receive_port}")
+        self.receive_socket = self.context.socket(zmq.REP)
+        self.receive_socket.bind(f"tcp://*:{self.receive_port}")
         while True:
             try:
-                # print("wait for client ...")
-                message = socket.recv()
-                # print("message from client:", message.decode('utf-8'))
+                message = self.receive_socket.recv()
                 self.message_queue.put(message.decode('utf-8'))
-                socket.send(message)
+                self.receive_socket.send(message)
             except Exception as e:
-                print('异常:',e)
+                print('异常:', e)
                 sys.exit()
 
-    def broadcast(self,message):
+    def broadcast(self, message):
+        if isinstance(message, dict):
+            message = json.dumps(message, ensure_ascii=False)
         self.message_queue.put(message)
 
+    def close(self):
+        """优雅关闭所有 ZeroMQ 资源"""
+        logger.info("正在关闭 MqServer...")
+        
+        # 关闭 socket
+        if self.publish_socket:
+            self.publish_socket.close()
+        if self.receive_socket:
+            self.receive_socket.close()
+
+        # 关闭 context
+        if self.context:
+            self.context.term()
+
+        logger.info("MqServer 已关闭")
+
     def start(self):
-        logger.info(f"启动发布通知的Server,port为{self.publish_port}")
+        logger.info(f"启动发布通知的Server, port为{self.publish_port}")
         threading.Thread(target=self.start_publish).start()
-        logger.info(f"启动接收通知的Server,port为{self.receive_port}")
+        logger.info(f"启动接收通知的Server, port为{self.receive_port}")
         threading.Thread(target=self.start_receive).start()
 
 
@@ -1263,6 +1368,236 @@ class MqConsumer():
         socket.connect(f"tcp://{host}:{port}")
         socket.setsockopt(zmq.SUBSCRIBE, ''.encode('utf-8'))  # 接收所有消息
         self.socket = socket
+    
+    def close(self):
+        if self.socket:
+            self.socket.close()
+        if self.context:
+            self.context.close()
 
     def receive(self):
-        return self.socket.recv().decode('utf-8')
\ No newline at end of file
+        return self.socket.recv().decode('utf-8')
+
+def backup_file():
+    file_path = sys.argv[1]
+    # 检查 bak 目录是否存在，不存在则创建
+    if not os.path.isdir("./bak"):
+        os.mkdir("./bak")
+
+    # 获取当前时间
+    day = datetime_tool.datetime.now().strftime("%Y%m%d_%H%M%S")
+    # 获取文件名
+    filename = os.path.basename(file_path)
+    # 构造目标文件名
+    backup_name = f"./bak/{filename}.bak.{day}"
+    # 复制并重命名文件
+    shutil.copy2(file_path, backup_name)
+    print(f"将文件备份为{backup_name}")
+
+
+def temp_delete_file():
+    file_path = sys.argv[1]
+    dir1 = datetime_tool.datetime.now().strftime("%Y%m%d%H%M%S")
+    targetDir = "/tmp/delete/"+dir1+"/"
+
+    mkdirCmd = "mkdir -p "+targetDir
+    os.system(mkdirCmd)
+    shutil.move(file_path, targetDir)
+
+def clear_look():
+    try:
+        file_path = sys.argv[1]
+        # 构造完整的 shell 命令
+        command = f"cat {file_path} | grep -v '#' | grep -v '^$' | grep -v '^[[:space:]]*$'"
+        
+        # 使用 subprocess 运行命令并捕获输出
+        result = subprocess.run(command, shell=True, check=True, text=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
+        
+        # 返回过滤后的内容
+        return result.stdout
+    except subprocess.CalledProcessError as e:
+        print(f"命令执行出错: {e.stderr}")
+        return ""
+    except Exception as e:
+        print(f"发生错误: {e}")
+        return ""
+
+def copy_file(file_path,target_path):
+    shutil.copy(file_path, target_path)
+    print(f"将文件复制到{target_path}")
+
+def uuid32():
+    # 生成 UUID
+    return uuid.uuid4().hex
+
+def uuid16():
+    # 生成 UUID4
+    uuid_str = str(uuid.uuid4())
+    # 去除 UUID 中的连接符 '-' 并截取前 16 位
+    return uuid_str.replace('-', '')[:16]
+
+def is_video(file_path):
+    file_path = file_path.lower()
+    video_extensions = ['.mp4', '.mkv', '.avi', '.flv', '.mov', '.wmv']
+    file_extension = os.path.splitext(file_path)[1]
+    return file_extension.lower() in video_extensions
+
+def is_image(file_path):
+    file_path = file_path.lower()
+    video_extensions = ['.png', '.jpeg', '.jpg', '.gif']
+    file_extension = os.path.splitext(file_path)[1]
+    return file_extension.lower() in video_extensions
+
+def is_music(file_path):
+    file_path = file_path.lower()
+    video_extensions = ['.mp3']
+    file_extension = os.path.splitext(file_path)[1]
+    return file_extension.lower() in video_extensions
+
+def is_mobile(user_agent):
+    user_agent = user_agent.lower()
+    mobile_patterns = [
+        r'Android',
+        r'iPhone',
+        r'iPad',
+        r'Windows Phone',
+        r'BlackBerry'
+    ]
+    for pattern in mobile_patterns:
+        if re.search(pattern, user_agent, re.IGNORECASE):
+            return True
+    return False
+
+class Img:
+    @staticmethod
+    def compress(image, target_size_kb=1024, step=5, min_quality=20,output_path=None) -> bytes:
+        """
+        image: PIL.Image.Image or str
+        将图片压缩到目标大小以内，返回压缩后的字节流
+        """
+        if isinstance(image, Image.Image):
+            img = image
+        elif isinstance(image, str):
+            img = Image.open(image)
+        else:
+            raise ValueError("image must be a PIL.Image.Image or a string path")
+
+        output = io.BytesIO()
+        # PNG 转 JPEG
+        img_format = img.format if img.format else "JPEG"
+        if img_format.upper() == "PNG":
+            img_format = "JPEG"
+            img = img.convert("RGB")  # PNG 转 JPEG 必须转 RGB
+
+        # 先尝试原图保存
+        img.save(output, format=img_format, quality=95, optimize=True)
+        if len(output.getvalue()) / 1024 <= target_size_kb:
+            return output.getvalue()
+
+        # 循环压缩
+        quality = 95
+        while quality >= min_quality:
+            output.seek(0)
+            output.truncate(0)
+            img.save(output, format=img_format, quality=quality, optimize=True)
+            size_kb = len(output.getvalue()) / 1024
+            if size_kb <= target_size_kb:
+                break
+            quality -= step
+        
+        if output_path:
+            with open(output_path, "wb") as f:
+                f.write(output.getvalue())
+        return output.getvalue()
+
+    @staticmethod
+    def cut(image,ratio,output_path=None) -> Image.Image:
+        """
+        image: PIL.Image.Image or str
+        ration is a tuple with four elements: left, top, right, bottom，percent of height and width
+        """
+        if isinstance(image, Image.Image):
+            img = image
+        elif isinstance(image, str):
+            img = Image.open(image)
+        else:
+            raise ValueError("image must be a PIL.Image.Image or a string path")
+        
+        height = img.height
+        width = img.width
+        left = 0 if ratio[0] is None or ratio[0] == -1 or ratio[0] == 0 else width * ratio[0]
+        top = 0 if ratio[1] is None or ratio[1] == -1  or ratio[1] == 0 else height * ratio[1]
+        right = width if ratio[2] is None or ratio[2] == -1 else width * ratio[2]
+        bottom = height if ratio[3] is None or ratio[3] == -1 else height * ratio[3]
+        new_image = img.crop((left, top, right, bottom))
+        if output_path:
+            new_image.save(output_path)
+        return new_image
+    
+    
+class DubboTool:
+    
+    def __init__(self, host: str, port: int, timeout: int = 10):
+        import telnetlib
+        self.host = host
+        self.port = port
+        self.timeout = timeout
+        self.tn = telnetlib.Telnet(self.host, self.port, self.timeout)
+        # 等待 dubbo 提示符
+        self.tn.read_until(b"dubbo>", timeout=self.timeout)
+
+    def new(self, service: str):
+        return DubboServiceProxy(self, service)
+
+    def invoke(self, service: str, method: str, params: dict):
+        """执行 telnet 调用"""
+        cmd = f'invoke {service}.{method}({json.dumps(params, ensure_ascii=False)})\n'
+        self.tn.write(cmd.encode("utf-8"))
+
+        result = self.tn.read_until(b"dubbo>", timeout=self.timeout).decode("utf-8")
+
+        # 清洗掉命令回显和提示符
+        cleaned = re.sub(r"^.*?\)\r?\n", "", result, flags=re.S).strip()
+        cleaned = cleaned.replace("dubbo>", "").strip()
+
+        # 尝试转 JSON
+        try:
+            return json.loads(cleaned)
+        except Exception:
+            return cleaned
+
+    def close(self):
+        self.tn.close()
+
+class DubboServiceProxy:
+    """代理对象：调用方法时转成 telnet 命令"""
+
+    def __init__(self, client: DubboTool, service: str):
+        self._client = client
+        self._service = service
+
+    def __getattr__(self, method: str):
+        def _method(params: dict):
+            return self._client.invoke(self._service, method, params)
+
+        return _method
+    
+
+class DatetimeTool:
+    def __init__(self):
+        pass
+
+    @staticmethod
+    def next_month_time(begin_time,return_str=True):
+        from datetime import relativedelta
+        if isinstance(begin_time,str):
+            begin_time = datetime_tool.datetime.strptime(begin_time, "%Y-%m-%d %H:%M:%S")
+        one_month_later = begin_time + relativedelta(months=1)
+        if return_str:
+            return one_month_later.strftime("%Y-%m-%d %H:%M:%S")
+        else:
+            return one_month_later
+    
+    @staticmethod
+    def now_str(format="%Y-%m-%d %H:%M:%S"):
+        return datetime_tool.datetime.now().strftime(format)
\ No newline at end of file
